@@ -17,7 +17,7 @@ from sklearn.neural_network import MLPRegressor
 jax.config.update("jax_enable_x64", True)
 jax.config.update("jax_platform_name", "cpu")
 
-SEED = 123  # Hard coded for reproducible predictions
+# Hard-coded lower and upper limit for kurtosis
 MIN_K = -3 / 7
 MAX_K = 10
 
@@ -324,6 +324,37 @@ def params_to_rd(params, mask=None):
     return rd
 
 
+def params_to_fa(params, mask=None):
+    """Compute fractional anisotropy.
+
+    Parameters
+    ----------
+    params : numpy.ndarray
+        Floating-point array with shape (..., 22).
+    mask : numpy.ndarray, optional
+        Boolean array.
+
+    Returns
+    -------
+    numpy.ndarray
+    """
+    if mask is None:
+        mask = np.ones(params.shape[0:-1]).astype(bool)
+    evals, _ = np.linalg.eigh(np.nan_to_num(params_to_D(params[mask])))
+    avg_evals = np.mean(evals, axis=-1)
+    fa = np.zeros(mask.shape)
+    fa[mask] = np.sqrt(
+        1.5
+        * (
+            (evals[..., 0] - avg_evals) ** 2
+            + (evals[..., 1] - avg_evals) ** 2
+            + (evals[..., 2] - avg_evals) ** 2
+        )
+        / np.sum(evals ** 2, axis=-1)
+    )
+    return fa
+
+
 @jax.jit
 def _akc(params, vs):
     """Compute apparent kurtosis coefficients along unit vectors `vs`.
@@ -649,7 +680,7 @@ def _nlls_fit(data, design_matrix, mask=None):
 
 @jax.jit
 def _akc_mask(W, vs, mask):
-    """Compute a mask based on 0 < AKC < 10 along all directions in `vs`.
+    """Compute a mask based on condition AKC > 0 along all directions in `vs`.
 
     Parameters
     ----------
@@ -667,12 +698,11 @@ def _akc_mask(W, vs, mask):
     akc_mask = np.ones(mask.shape)
     for v in vs:
         akc_mask *= (v.T @ (v.T @ W @ v) @ v) > 0
-        akc_mask *= (v.T @ (v.T @ W @ v) @ v) < 10
     akc_mask *= mask
     return akc_mask
 
 
-def _predict(data, m, akc_mask, mask=None):
+def _predict(data, m, akc_mask, seed, mask=None):
     """Train a multilayer perceptron to predict `m` from `data`.
 
     Parameters
@@ -683,6 +713,9 @@ def _predict(data, m, akc_mask, mask=None):
         Floating-point array.
     akc_mask : numpy.ndarray
         Boolean array.
+    seed : int
+        Seed for pseudo-random number generation to initialize neural network
+        weights.
     mask : numpy.ndarray, optional
         Boolean array.
 
@@ -695,7 +728,9 @@ def _predict(data, m, akc_mask, mask=None):
         mask = np.ones(data.shape[0:-1]).astype(bool)
     X = data[akc_mask]
     y = np.clip(np.nan_to_num(m[akc_mask]), MIN_K, MAX_K)
-    reg = MLPRegressor(random_state=SEED, hidden_layer_sizes=(50, 50)).fit(X, y)
+    reg = MLPRegressor(
+        hidden_layer_sizes=(20, 20), max_iter=int(1e3), random_state=seed,
+    ).fit(X, y)
     R2 = reg.score(X, y)
     m_pred = np.zeros(mask.shape)
     m_pred[mask] = reg.predict(data[mask])
@@ -996,12 +1031,13 @@ def _reg_nlls_fit(
     return params, fit_status
 
 
-def fit(data, bvals, bvecs, mask=None, alpha=None, quiet=False):
+def fit(data, bvals, bvecs, mask=None, alpha=None, seed=123, quiet=False):
     """Estimate diffusion and kurtosis tensor elements.
 
     This function does the following:
 
-        1. Remove infinities, nans, and negative values, and normalize data.
+        1. Remove infinities, nans, and negative values, and scale data and
+           b-values.
         2. Estimate DKI model parameters using standard NLLS.
         3. Train a multilayer perceptron to predict kurtosis measures from data
            in voxels where the apparent kurtosis coefficient estimated by the
@@ -1017,15 +1053,17 @@ def fit(data, bvals, bvecs, mask=None, alpha=None, quiet=False):
     data : numpy.ndarray
         Floating-point array with shape (..., number of acquisitions).
     bvals : numpy.ndarray
-        Floating-point array with shape (number of acquisitions,). Must be in
-        units of ms/μm^2!
+        Floating-point array with shape (number of acquisitions,).
     bvecs : numpy.ndarray
         Floating-point array with shape (number of acquisitions, 3).
     mask : numpy.ndarray, optional
         Boolean array with the same shape as `data` without the last dimension.
     alpha : float, optional
         Constant controlling regularization term magnitudes in the objective
-        function. 
+        function.
+    seed : int
+        Seed for pseudo-random number generation to initialize neural network
+        weights.
     quiet : bool
         Whether not to print messages about computation progress.
 
@@ -1074,10 +1112,12 @@ def fit(data, bvals, bvecs, mask=None, alpha=None, quiet=False):
 
     data[np.isinf(data)] = np.nan
     data[np.isnan(data)] = 0
-    C = np.nanmean(data[..., np.where(bvals == np.min(bvals))])
-    data /= C
+    C_data = np.mean(data[..., np.where(bvals == np.min(bvals))])
+    data /= C_data
     min_signal = np.finfo(float).resolution
     data[data < min_signal] = min_signal
+    C_bvals = np.mean(bvals)
+    bvals /= C_bvals
 
     if not quiet:
         print("Fitting DKI to data with standard NLLS")
@@ -1088,25 +1128,25 @@ def fit(data, bvals, bvecs, mask=None, alpha=None, quiet=False):
     if not quiet:
         print("Training a MLP to predict MK")
     mk = np.clip(params_to_mk(nlls_params, mask), MIN_K, MAX_K)
-    mk_pred, R2 = _predict(data, mk, akc_mask, mask)
+    mk_pred, R2 = _predict(data, mk, akc_mask, seed, mask)
     if not quiet:
         print(f"R^2 on training data for MK = {R2}")
     if not quiet:
         print("Training a MLP to predict MTK")
     mtk = np.clip(_mtk(nlls_params, mask), MIN_K, MAX_K)
-    mtk_pred, R2 = _predict(data, mtk, akc_mask, mask)
+    mtk_pred, R2 = _predict(data, mtk, akc_mask, seed, mask)
     if not quiet:
         print(f"R^2 on training data for MTK = {R2}")
     if not quiet:
         print("Training a MLP to predict AK")
     ak = np.clip(params_to_ak(nlls_params, mask), MIN_K, MAX_K)
-    ak_pred, R2 = _predict(data, ak, akc_mask, mask)
+    ak_pred, R2 = _predict(data, ak, akc_mask, seed, mask)
     if not quiet:
         print(f"R^2 on training data for AK = {R2}")
     if not quiet:
         print("Training a MLP to predict RK")
     rk = np.clip(params_to_rk(nlls_params, mask), MIN_K, MAX_K)
-    rk_pred, R2 = _predict(data, rk, akc_mask, mask)
+    rk_pred, R2 = _predict(data, rk, akc_mask, seed, mask)
     if not quiet:
         print(f"R^2 on training data for RK = {R2}")
 
@@ -1145,6 +1185,10 @@ def fit(data, bvals, bvecs, mask=None, alpha=None, quiet=False):
         quiet,
     )
 
+    params[..., 0] += np.log(C_data)
+    params[..., 1:7] /= C_bvals
+    params[..., 7::] /= C_bvals ** 2
+
     return params, fit_status, mk_pred, ak_pred, rk_pred
 
 
@@ -1161,7 +1205,7 @@ if __name__ == "__main__":
         "data", help="path of a NIfTI file with diffusion-weighted data",
     )
     parser.add_argument(
-        "bvals", help="path of a text file with b-values in units of s/mm^2",
+        "bvals", help="path of a text file with b-values",
     )
     parser.add_argument(
         "bvecs", help="path of a text file with b-vectors",
@@ -1174,43 +1218,48 @@ if __name__ == "__main__":
         ),
     )
     parser.add_argument(
-        "-md", help="path of a file in which to save the mean diffusivity map"
+        "-md", help="path of a NIfTI file in which to save the mean diffusivity map"
     )
     parser.add_argument(
-        "-ad", help="path of a file in which to save the axial diffusivity map",
+        "-ad", help="path of a NIfTI file in which to save the axial diffusivity map",
     )
     parser.add_argument(
-        "-rd", help="path of a file in which to save the radial diffusivity map",
+        "-rd", help="path of a NIfTI file in which to save the radial diffusivity map",
     )
     parser.add_argument(
-        "-mk", help="path of a file in which to save the mean kurtosis map"
+        "-fa",
+        help="path of a NIfTI file in which to save the fractional anisotropy map",
     )
     parser.add_argument(
-        "-ak", help="path of a file in which to save the axial kurtosis map",
+        "-mk", help="path of a NIfTI file in which to save the mean kurtosis map"
     )
     parser.add_argument(
-        "-rk", help="path of a file in which to save the radial kurtosis map",
+        "-ak", help="path of a NIfTI file in which to save the axial kurtosis map",
     )
     parser.add_argument(
-        "-s0", help="path of a file in which to save the estimated signal at b=0",
+        "-rk", help="path of a NIfTI file in which to save the radial kurtosis map",
+    )
+    parser.add_argument(
+        "-s0", help="path of a NIfTI file in which to save the estimated signal at b=0",
     )
     parser.add_argument(
         "-mk_pred",
-        help="path of a file in which to save the predicted mean kurtosis map",
+        help="path of a NIfTI file in which to save the predicted mean kurtosis map",
     )
     parser.add_argument(
         "-ak_pred",
-        help="path of a file in which to save the predicted axial kurtosis map",
+        help="path of a NIfTI file in which to save the predicted axial kurtosis map",
     )
     parser.add_argument(
         "-rk_pred",
-        help="path of a file in which to save the predicted radial kurtosis map",
+        help="path of a NIfTI file in which to save the predicted radial kurtosis map",
     )
     parser.add_argument(
-        "-fit_status", help="path of a file in which to save the fit status map",
+        "-status", help="path of a NIfTI file in which to save the fit status map",
     )
     parser.add_argument(
-        "-params", help="path of a file in which to save the parameters",
+        "-params",
+        help="path of a NIfTI file in which to save the estimated parameters",
     )
     args = parser.parse_args()
 
@@ -1242,6 +1291,8 @@ if __name__ == "__main__":
         nib.save(nib.Nifti1Image(params_to_ad(params, mask) * 1e-3, affine), args.ad)
     if args.rd:
         nib.save(nib.Nifti1Image(params_to_rd(params, mask) * 1e-3, affine), args.rd)
+    if args.fa:
+        nib.save(nib.Nifti1Image(params_to_fa(params, mask) * 1e-3, affine), args.fa)
     if args.mk:
         nib.save(nib.Nifti1Image(params_to_mk(params, mask), affine), args.mk)
     if args.ak:
@@ -1249,7 +1300,7 @@ if __name__ == "__main__":
     if args.rk:
         nib.save(nib.Nifti1Image(params_to_rk(params, mask), affine), args.rk)
     if args.s0:
-        nib.save(nib.Nifti1Image(np.exp(params[..., 0] + np.log(C)), affine), args.s0)
+        nib.save(nib.Nifti1Image(np.exp(params[..., 0]), affine), args.s0)
     if args.mk_pred:
         nib.save(nib.Nifti1Image(mk_pred, affine), args.mk_pred)
     if args.ak_pred:
